/*****************************************************************************************************************************
--
--    File Name    : APB_WRAPPER.v 

--    Description  : This module is APB slave to update registers.
--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1 ns/100 ps

module APB_WRAPPER 
 #(
/******************************************************************************************************************************/
/***********************************************Parameters*********************************************************************/
/******************************************************************************************************************************/
	parameter  APB_DWIDTH                 = 32,
	parameter  APB_AWIDTH                 = 32,
    parameter  g_B_C_SAT_CONSTANT_WIDTH   = 8,
    parameter  g_HUE_CONSTANT_WIDTH       = 9,
    parameter  g_SIN_COS_DWIDTH           = 10
 )(
/******************************************************************************************************************************/
/***********************************************Module IO**********************************************************************/
/******************************************************************************************************************************/
   // APB interface from MSS
	input  wire				       PCLK,
	input  wire				       PRESETN,
	input  wire [APB_AWIDTH - 1:0] PADDR,
	input  wire	     		       PWRITE,
	input  wire	 			       PSEL,
	output wire				       PREADY,
	input  wire			           PENABLE,
	input  wire [APB_DWIDTH - 1:0] PWDATA,
	output reg [APB_DWIDTH - 1:0]  PRDATA,
	output wire 				   PSLVERR,

    output reg [7:0]               sin_mem_addr,
    output reg [7:0]               cos_mem_addr,
    output reg                     sin_write_en,
    output reg                     cos_write_en,
    output reg [(g_SIN_COS_DWIDTH-1):0] sin_value,
    output reg [(g_SIN_COS_DWIDTH-1):0] cos_value,

     //Brightness, Contrast, saturation, hue and sharpness register values
     output reg signed [(g_B_C_SAT_CONSTANT_WIDTH-1):0] brightness_const,
     output reg signed [(g_B_C_SAT_CONSTANT_WIDTH-1):0] contrast_const,
     output reg signed [(g_B_C_SAT_CONSTANT_WIDTH-1):0] saturation_const,
     output reg signed     [(g_HUE_CONSTANT_WIDTH-1):0] hue_const ,
     output reg        [7:0]                            K_sharpness,
     //Alpha Blending Register Values
     output reg        [10:0]                           AB_Image_PosX,
     output reg        [10:0]                           AB_Image_PosY,
     output reg        [7 :0]                           Global_Alpha,
     output reg        [31 :0]                          AB_Object_Address

        
  );   

	//Net assignment
    reg [11:0] sin_mem_addr_reg,cos_mem_addr_reg,sin_mem_addr_reg1,cos_mem_addr_reg1;
	


	assign PREADY  = 1'b1;
	assign PSLVERR = 1'b0;



/******************************************************************************************************************************/
/********************************Process to adjust Brightness and Configure the Firmware Version*******************************/
/******************************************************************************************************************************/

    always @(posedge PCLK or negedge PRESETN)
    begin
     if (!PRESETN)
     begin
      brightness_const <= {g_B_C_SAT_CONSTANT_WIDTH{1'b0}};
        PRDATA           <= 32'h00020001;  
	   end
	   else
     begin
		  if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h600)) 
		  begin
		   brightness_const <= PWDATA[(g_B_C_SAT_CONSTANT_WIDTH-1):0];
            PRDATA           <= 32'h00020001;  
		  end
	   end
   end
  

/******************************************************************************************************************************/
/********************************Process to adjust Contrast of the Display*****************************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
      contrast_const <= {{(g_B_C_SAT_CONSTANT_WIDTH-3){1'b0}}, 3'd1};
	 end
	 else
     begin
		  if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h604)) 
		  begin
		   contrast_const   <= PWDATA[(g_B_C_SAT_CONSTANT_WIDTH-1):0];
		  end
	  end
  end
  


/******************************************************************************************************************************/
/********************************Process to adjust Saturation of the Display*****************************************************/
/******************************************************************************************************************************/
always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
      saturation_const <= {{(g_B_C_SAT_CONSTANT_WIDTH-3){1'b0}}, 3'd1};
	 end
	 else
     begin
		   if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h608)) 
		  begin
		       saturation_const <= PWDATA[(g_B_C_SAT_CONSTANT_WIDTH-1):0];
		  end
	  end
  end
  
 


/******************************************************************************************************************************/
/********************************Process to adjust Hue of the Display**********************************************************/
/******************************************************************************************************************************/
always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
      hue_const        <= {g_HUE_CONSTANT_WIDTH{1'b0}};
	 end
	 else
     begin
		  if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h60C)) 
		  begin
		        hue_const        <= PWDATA[(g_HUE_CONSTANT_WIDTH-1):0];
		  end
	  end
  end
 


/******************************************************************************************************************************/
/********************************Process to adjust Sharpness of the Display****************************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	       K_sharpness      <= 8'd0; 
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h610)) 
			 K_sharpness      <= PWDATA[7:0];
	 end
 end
 



/******************************************************************************************************************************/
/********************************Process to initialize Sine Look Up table *****************************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	 sin_value        <= {g_SIN_COS_DWIDTH{1'b1}};
	 sin_mem_addr     <= 8'd0;
	  sin_write_en     <= 1'b0;
     sin_mem_addr_reg <= 12'd0;
     sin_mem_addr_reg1 <= 12'd0;
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && ((PADDR[11:0] >= 12'h300) && (PADDR[11:0] < 12'h5D4))) 
		begin
		sin_value        <= PWDATA[(g_SIN_COS_DWIDTH-1):0];
		sin_mem_addr_reg     <= PADDR[11:0];
        sin_mem_addr_reg1 <= ((sin_mem_addr_reg - 12'h300)/12'd4);
        sin_mem_addr <= sin_mem_addr_reg1 [7:0];
		sin_write_en     <= 1'b1;
    	end
        else
        begin
		sin_value        <= {g_SIN_COS_DWIDTH{1'b1}};
		sin_mem_addr     <= 8'd0;
        sin_mem_addr_reg <= 12'd0;
        sin_mem_addr_reg1 <= 12'd0;
        sin_write_en <=1'b0;
        end
	 end
 end
 


/******************************************************************************************************************************/
/********************************Process to initialize Cosine Look Up table *****************************************************/
/******************************************************************************************************************************/
   always @(posedge PCLK or negedge PRESETN)
   begin
     if (!PRESETN)
     begin
	 cos_value        <= {g_SIN_COS_DWIDTH{1'b1}};
	 cos_mem_addr     <= 8'd0;	
     cos_write_en     <= 1'b0;	
     cos_mem_addr_reg <= 12'd0; 
     cos_mem_addr_reg1 <= 12'd0;
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && ((PADDR[11:0] >= 12'h000) && (PADDR[11:0] < 12'h2D4)))  
		begin
		cos_value        <= PWDATA[(g_SIN_COS_DWIDTH-1):0];
        cos_mem_addr_reg <= PADDR[11:0];
		cos_mem_addr_reg1     <= cos_mem_addr_reg;
        cos_mem_addr <= cos_mem_addr_reg1[9:2];
		cos_write_en     <= 1'b1; 
		end
        else
        begin
		cos_value        <= {g_SIN_COS_DWIDTH{1'b1}};
		cos_mem_addr     <= 8'd0;
        cos_mem_addr_reg <= 12'd0;
		cos_write_en     <= 1'b0;
        cos_mem_addr_reg1 <= 12'd0;
        end
	 end
 end





/******************************************************************************************************************************/
/********************************Process to load Image X-Coordinate for Alpha Blending*****************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	       AB_Image_PosX      <= 11'd1; 
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h61C)) 
			 AB_Image_PosX      <= PWDATA[10:0];
	 end
 end



/******************************************************************************************************************************/
/********************************Process to load Image Y-Coordinate for Alpha Blending*****************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	       AB_Image_PosY      <= 11'd1; 
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h620)) 
			 AB_Image_PosY      <= PWDATA[10:0];
	 end
 end



/******************************************************************************************************************************/
/********************************Process to adjust Global Alpha for Alpha Blending*********************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	       Global_Alpha      <= 8'hFF; 
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h624)) 
			 Global_Alpha      <= PWDATA[7:0];
	 end
 end



/******************************************************************************************************************************/
/********************************Process to load Object Address for Alpha Blending*********************************************/
/******************************************************************************************************************************/
 always @(posedge PCLK or negedge PRESETN)
 begin
     if (!PRESETN)
     begin
	       AB_Object_Address      <= 32'hA0000000; 
	 end
	 else
	 begin
		if ((PENABLE == 1'b1) && (PWRITE == 1'b1) && (PSEL == 1'b1) && (PADDR[11:0] == 12'h628)) 
			 AB_Object_Address      <= PWDATA[31:0];
	 end
 end

 
endmodule