/*****************************************************************************************************************************
--
--    File Name    : cam_data_pack.v

--    Description  : This module packs 8-bit data out from Camera to 32-bit words to be written into DDR
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module camdata_pack #(
                       /******************************************************************************************************************************/
                       /***********************************************Parameters*********************************************************************/
                       /******************************************************************************************************************************/
                       parameter g_CAM_DATA_WIDTH = 8,
                       parameter g_DDR_DATA_WIDTH = 32,
                       parameter g_DDR_ADDRESS_WIDTH = 32,
                       parameter g_X_Y_WIDTH = 2,
                       parameter g_Horizontal_Resolution = 11'd1280)
                     (
                      /******************************************************************************************************************************/
                      /***********************************************Module IO**********************************************************************/
                      /******************************************************************************************************************************/    
                      input wire CAM_CLOCK_I,
                      input wire DDR_CLOCK_I,
                      input wire RESET_N_I,
                      input wire[(g_CAM_DATA_WIDTH - 1) : 0] pixel_in_i,
                      input wire pixel_in_vld_i,
                      input wire frame_done_i,
                      input wire frame_start_i,
                      input wire line_done_i,
                      input wire line_start_i,
                      input wire[(g_DDR_ADDRESS_WIDTH - 1) : 0] display_ddr_address_i,
                      output wire[(g_DDR_ADDRESS_WIDTH - 1) : 0] start_ddr_address_o,
                      output reg[(g_DDR_DATA_WIDTH - 1) : 0] pixel_out_o,
                      output reg pixel_vld_o,
                      output wire ddr_start_proc_o,
                      output reg[(g_DDR_ADDRESS_WIDTH - 1) : 0] DC_FRM_BUFF_DDR_ADDR_o,
                      output reg display_control_en_o);
  //Registers declarations  
  reg[(g_X_Y_WIDTH - 1) : 0] s_r_frame_count;
  reg[1 : 0] s_r_input_count;
  reg[(g_DDR_DATA_WIDTH - 1) : 0] s_r_data_pack0;
  reg[10: 0] s_r_write_address;
  reg[10 : 0] s_r_read_address;
  wire s_r_we;
  reg s_r_start_read;
  wire[(g_DDR_DATA_WIDTH - 1) : 0] pixel_out_int;
  reg pixel_vld_int;
  reg s_r_frame_done;
  reg s_r_frame_start;
  reg s_r_line_done;
  reg s_r_line_done_d;
  reg s_r_line_done_2d;
  reg s_r_line_start;
  wire s_r_line_done_dtc;
  wire s_r_line_start_dtc;
  wire s_r_frame_done_dtc;
  wire s_r_frame_start_dtc;
  reg[(g_DDR_ADDRESS_WIDTH - 1) : 0] start_ddr_address_reg;
  reg[(g_DDR_ADDRESS_WIDTH - 1) : 0] start_ddr_address_reg_plus4k;
  reg[(g_DDR_ADDRESS_WIDTH - 1) : 0] display_ddr_addr_d;
  reg s_r_nxt_frm_eq_displ_frm;
  reg[10 : 0] s_r_write_address_reg;
  reg s_r_we_reg,
      s_r_we_reg1;
  wire [10:0] cdcfifo_Hwidth;
  assign start_ddr_address_o = start_ddr_address_reg;
  assign cdcfifo_Hwidth = (g_Horizontal_Resolution / 4);
  /******************************************************************************************************************************/
  /**************************Process to Register Input data*********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_frame_done <= 1'b0;
          s_r_frame_start <= 1'b0;
          s_r_line_start <= 1'b0;
        end
      else
        begin
          s_r_frame_done <= frame_done_i;
          s_r_frame_start <= frame_start_i;
          s_r_line_start <= line_start_i;
        end
    end
  assign s_r_frame_done_dtc = ~s_r_frame_done & frame_done_i;
  assign s_r_frame_start_dtc = ~s_r_frame_start & frame_start_i;
  assign s_r_line_start_dtc = ~s_r_line_start & line_start_i;
  always
    @(posedge (DDR_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_line_done <= 1'b0;
          s_r_line_done_d <= 1'b0;
          s_r_line_done_2d <= 1'b0;
        end
      else
        begin
          s_r_line_done <= line_done_i;
          s_r_line_done_d <= s_r_line_done;
          s_r_line_done_2d <= s_r_line_done_d;
        end
    end
  assign s_r_line_done_dtc = ~s_r_line_done_2d & s_r_line_done_d;
  /******************************************************************************************************************************/
  /**************************Process to Generate Frame Count*********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          start_ddr_address_reg_plus4k <= 'h0;
          s_r_nxt_frm_eq_displ_frm <= 1'b0;
	  display_ddr_addr_d <= 'h0;
        end
      else
        begin
          start_ddr_address_reg_plus4k <= (start_ddr_address_reg + 24'h400000);
          display_ddr_addr_d <= display_ddr_address_i;
          s_r_nxt_frm_eq_displ_frm <= (start_ddr_address_reg_plus4k == display_ddr_addr_d);
        end
    end

  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_frame_count <= 'b0;
        end
      else
        begin
          if (s_r_frame_done_dtc == 1'b1)
            begin
              if (s_r_nxt_frm_eq_displ_frm)
                s_r_frame_count <= s_r_frame_count;
              else
                s_r_frame_count <= s_r_frame_count + 1'b1;
            end
        end
    end
  /******************************************************************************************************************************/
  /**************************Process to Generate Display Control Enable**********************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          display_control_en_o <= 1'b0;
        end
      else
        begin
          if (s_r_frame_count == 'b01)
            begin
              display_control_en_o <= 1'b1;
            end
        end
    end
  /******************************************************************************************************************************/
  /**************************Process to Generate DDR address*********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          start_ddr_address_reg <= 32'hAD000000;
        end
      else
        begin
          case (s_r_frame_count[1 : 0])
            2'b00:
              begin
                start_ddr_address_reg <= 32'hAD000000;
              end
            2'b01:
              begin
                start_ddr_address_reg <= 32'hAD400000;
              end
            2'b10:
              begin
                start_ddr_address_reg <= 32'hAD800000;
              end
            2'b11:
              begin
                start_ddr_address_reg <= 32'hADC00000;
              end
            default:
              begin
                start_ddr_address_reg <= 32'hAD000000;
              end
          endcase
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          DC_FRM_BUFF_DDR_ADDR_o <= 32'hADC00000;
        end
      else
        begin
          case (start_ddr_address_reg)
            32'hAD400000:
              DC_FRM_BUFF_DDR_ADDR_o <= 32'hAD000000;
            32'hAD800000:
              DC_FRM_BUFF_DDR_ADDR_o <= 32'hAD400000;
            32'hADC00000:
              DC_FRM_BUFF_DDR_ADDR_o <= 32'hAD800000;
            32'hAD000000:
              DC_FRM_BUFF_DDR_ADDR_o <= 32'hADC00000;
            default:
              DC_FRM_BUFF_DDR_ADDR_o <= 32'hADC00000;
          endcase
        end
    end
  /******************************************************************************************************************************/
  /**************************Process to Pack 8-bit data to 32-bit data***********************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_input_count <= 2'd3;
        end
      else
        begin
          if ((s_r_input_count == 2'd3 && pixel_in_vld_i == 1'b1) || s_r_line_start_dtc == 1'b1 || s_r_frame_start_dtc == 1'b1)
            begin
              s_r_input_count <= 2'd0;
            end
          else
            begin
              if (pixel_in_vld_i == 1'b1)
                begin
                  s_r_input_count <= s_r_input_count + 1'b1;
                end
            end
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_data_pack0 <= 'd0;
        end
      else
        begin
          if (s_r_line_start_dtc == 1'b1 || s_r_frame_start_dtc == 1'b1)
            begin
              s_r_data_pack0 <= 'd0;
            end
          else
            begin
              if (pixel_in_vld_i == 1'b1)
                begin
                  s_r_data_pack0 <= { pixel_in_i, s_r_data_pack0[31 : 8] };
                end
            end
        end
    end
  /******************************************************************************************************************************/
  /**************************Process to Generate Write Request*******************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_write_address_reg <= 11'd0;
          s_r_we_reg <= 1'd0;
        end
      else
        begin
          if (s_r_line_start_dtc == 1'b1)
            begin
              s_r_write_address_reg <= 11'd0;
            end
          else
            if (s_r_input_count == 2'd2 && pixel_in_vld_i == 1'b1 && s_r_write_address < (cdcfifo_Hwidth))
              begin
                s_r_write_address_reg <= s_r_write_address_reg + 1'b1;
                s_r_we_reg <= 1'd1;
              end
            else
              begin
                s_r_we_reg <= 1'd0;
              end
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_write_address <= 11'd0;
          s_r_we_reg1 <= 1'd1;
        end
      else
        begin
          s_r_write_address <= s_r_write_address_reg;
          s_r_we_reg1 <= s_r_we_reg;
        end
    end

  assign s_r_we = s_r_we_reg & ~s_r_we_reg1;
  /******************************************************************************************************************************/
  /**************************Process to Generate Read Request********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (DDR_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_start_read <= 1'b0;
        end
      else
        begin
          if (s_r_read_address <= ((cdcfifo_Hwidth) - 1))
            begin
              s_r_start_read <= 1'b1;
            end
          else
            begin
              s_r_start_read <= 1'b0;
            end
        end
    end
  /******************************************************************************************************************************/
  /**************************Process to Generate Read Address********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (DDR_CLOCK_I) or
      negedge (RESET_N_I))
    begin
      if (!RESET_N_I)
        begin
          s_r_read_address <= 11'h7FF;
        end
      else
        begin
          if (s_r_line_done_dtc == 1'b1)
            begin
              s_r_read_address <= 11'd0;
            end
          else
            begin
              if (s_r_read_address <= ((cdcfifo_Hwidth) - 1))
                begin
                  s_r_read_address <= s_r_read_address + 1'b1;
                end
            end
        end
    end
  
 
  //--------cdcfifo
  fifocdc #(.ADDR_WIDTH(11), .DATA_WIDTH(g_DDR_DATA_WIDTH)) cdcfifo_0(
                                                                     // Inputs
                                                                     .k_lim((cdcfifo_Hwidth)),
                                                                     .wrrstn(RESET_N_I),
                                                                     .wrsrst(!RESET_N_I),
                                                                     .wrclk(CAM_CLOCK_I),
                                                                     .wrreq(s_r_we),
                                                                     .wrdata(s_r_data_pack0),
                                                                     .rdrstn(RESET_N_I),
                                                                     .rdclk(DDR_CLOCK_I),
                                                                     .rdsrst(!RESET_N_I),
                                                                     .rdreq(s_r_start_read),
                                                                     // Outputs
                                                                     .wrfull(),
                                                                     .wrusedw(),
                                                                     .rddata(pixel_out_int),
                                                                     .rdempty(),
                                                                     .rdusedw());
  assign ddr_start_proc_o = (s_r_read_address == ((g_Horizontal_Resolution / 4) - 3) || s_r_read_address == ((g_Horizontal_Resolution / 4) - 2)) ? 1'b1 : 1'b0;
  always
    @(posedge DDR_CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          pixel_out_o <= 'h0;
          pixel_vld_o <= 1'b0;
          pixel_vld_int <= 1'b0;
        end
      else
        begin
          pixel_out_o <= pixel_out_int;
          pixel_vld_o <= pixel_vld_int;
          pixel_vld_int <= s_r_start_read;
        end
    end
endmodule